@@ -8,6 +8,9 @@ __all__ = ["get_data_file_path",
            "warning_on_one_line",
            "check_if_path_exists",
            "create_path",
+           "unit_to_string",
+           "quantity_to_string",
+           "convert_all_quantities_to_string",
            "Logger"
            ]
 
@@ -15,6 +18,7 @@ __all__ = ["get_data_file_path",
 from pkg_resources import resource_filename
 import os
 import contextlib
+from simtk import unit
 
 
 def get_data_file_path(relative_path):
@@ -96,6 +100,130 @@ def create_path(path):
     os.makedirs(str(path), exist_ok=True)
 
 
+def unit_to_string(input_unit):
+    """
+    Serialize a simtk.unit.Unit and return it as a string.
+
+    Function modified from the OpenFF Toolkit
+    (https://github.com/openforcefield/openforcefield/).
+
+    Parameters
+    ----------
+    input_unit : A simtk.unit
+        The unit to serialize
+
+    Returns
+    -------
+    unit_string : str
+        The serialized unit.
+    """
+
+    if input_unit == unit.dimensionless:
+        return "dimensionless"
+
+    # Decompose output_unit into a tuples of (base_dimension_unit, exponent)
+    unit_string = None
+    for unit_component in input_unit.iter_base_or_scaled_units():
+        unit_component_name = unit_component[0].name
+
+        # Convert, for example "elementary charge" --> "elementary_charge"
+        unit_component_name = unit_component_name.replace(' ', '_')
+        if unit_component[1] == 1:
+            contribution = '{}'.format(unit_component_name)
+        else:
+            contribution = '{}**{}'.format(unit_component_name,
+                                           int(unit_component[1]))
+        if unit_string is None:
+            unit_string = contribution
+
+        else:
+            unit_string += ' * {}'.format(contribution)
+
+    return unit_string
+
+
+def quantity_to_string(input_quantity):
+    """
+    Serialize a simtk.unit.Quantity to a string.
+
+    Function modified from the OpenFF Toolkit
+    (https://github.com/openforcefield/openforcefield/).
+
+    Parameters
+    ----------
+    input_quantity : simtk.unit.Quantity
+        The quantity to serialize
+
+    Returns
+    -------
+    output_string : str
+        The serialized quantity
+    """
+
+    import numpy as np
+
+    if input_quantity is None:
+        return None
+
+    unitless_value = input_quantity.value_in_unit(input_quantity.unit)
+
+    # The string representation of a numpy array doesn't have commas and
+    # breaks the parser, thus we convert any arrays to list here
+    if isinstance(unitless_value, np.ndarray):
+        unitless_value = list(unitless_value)
+
+    unit_string = unit_to_string(input_quantity.unit)
+    output_string = '{} * {}'.format(unitless_value, unit_string)
+
+    return output_string
+
+
+def convert_all_quantities_to_string(data_structure):
+    """
+    Traverses a data structure, attempting to convert all quantities
+    into strings.
+
+    Function modified from the OpenFF Toolkit
+    (https://github.com/openforcefield/openforcefield/).
+
+    Parameters
+    ----------
+    data_structure : dict
+        A hierarchical dict structured
+
+    Returns
+    -------
+    converted_data_structure : dict
+        A hierarchical dict structured with simtk.unit.Quantitys
+        converted to string
+    """
+    from copy import copy
+    from peleffy.forcefield.parameters import BaseParameterWrapper
+
+    data_structure = copy(data_structure)
+
+    if isinstance(data_structure, BaseParameterWrapper):
+        data_structure = dict(data_structure)
+
+    if isinstance(data_structure, dict):
+        for key, value in data_structure.items():
+            data_structure[key] = convert_all_quantities_to_string(value)
+        obj_to_return = data_structure
+
+    elif isinstance(data_structure, list):
+        for index, item in enumerate(data_structure):
+            data_structure[index] = convert_all_quantities_to_string(item)
+        obj_to_return = data_structure
+
+    elif isinstance(data_structure, unit.Quantity):
+        obj_to_return = quantity_to_string(data_structure)
+
+    else:
+        obj_to_return = data_structure
+
+    return obj_to_return
+
+
 class Logger(object):
     """
     It contains all the required methods to handle logging messages.
